require 'digest/sha1'
require 'base64'
require 'openssl'
module EcconnectRails
  module BaseModel
    extend ActiveSupport::Concern

    included do
      CURRENCY = [980, 643, 840, 978]
    end

    module ClassMethods
      def ecconnect_payment
        validates :merchant_id, :terminal_id, :currency_id, :amount, presence: true
        before_validation :set_terminal_id, :set_merchant_id, :set_locale, :set_currency_id
        before_create :set_tran_code
      end
    end

    def purchase_time
      self.created_at.strftime("%Y%m%d%H%M%S%z") if self.created_at
    end

    def currency
      I18n.t "ecconnect_rails.currency_#{self.currency_id}"
    end

    def request_data
      @request_data ||= make_request_data
    end

    def check_data
      @check_data ||= make_check_data
    end

    def signature
      @signature ||= sign(request_data)
    end

    def upc_amount 
      (self.amount*100).to_i
    end

    def code
      I18n.t "ecconnect_rails.codes.code_#{self.tran_code}" if self.tran_code
    end

    def ecconnect_json
      self.as_json(except: [:income_signature, :proxy_pan, :rrn, :sd, :xid, :approval_code], methods: [:purchase_time, :signature, :code])
    end

    def check_signature
      cert.public_key.verify OpenSSL::Digest::SHA1.new, Base64.decode64(self.income_signature), check_data
    end

    def update_by params
      self.update_attributes :proxy_pan => params[:ProxyPan],
                              :approval_code => params[:ApprovalCode],
                              :income_signature => params[:Signature],
                              :rrn => params[:Rrn],
                              :xid => params[:XID],
                              :email => params[:Email],
                              :sd => params[:SD],
                              :tran_code => params[:TranCode]
    end


  private
    def set_locale
      self.locale = I18n.locale.to_s
    end

    def set_currency_id
      self.currency_id = EcconnectRails.default_options[:currency_id]
    end

    def set_terminal_id
      self.terminal_id = EcconnectRails.default_options[:terminal_id]
    end

    def set_merchant_id
      self.merchant_id = EcconnectRails.default_options[:merchant_id]
    end

    def set_tran_code
      self.tran_code = nil
    end

    def sign data
      Base64.encode64(pem.sign(OpenSSL::Digest::SHA1.new, data)).gsub(/\n/, '')
    end

    def pem
      OpenSSL::PKey::RSA.new File.read EcconnectRails.default_options[:pem_path]
    end

    def cert
      OpenSSL::X509::Certificate.new File.read EcconnectRails.default_options[:cert_path]
    end

    #MerchantId;TerminalId;PurchaseTime;OrderId,Delay;Xid;CurrencyId,AltCurrencyId;Amount,AltAmount;SessionData;TranCode;ApprovalCode;
    def make_check_data
      data = ""
      data << (self.merchant_id.nil? ? "" : "#{self.merchant_id}")
      data << (self.terminal_id.nil? ? ";" : ";#{self.terminal_id}")
      data << ";#{self.purchase_time}"
      data << ";#{self.order_id}"
      data << (self.delay.nil? ? "" : ", #{self.delay}")
      data << (self.xid.nil? ? ";" : ";#{self.xid}")
      if self.currency_id.nil?
        data << ";"
      else
        data << ";#{self.currency_id}"
        data << (self.alt_currency_id.nil? ? "" : ", #{self.alt_currency_id}")
      end
      if self.amount.nil?
        data << ";"
      else
        data << ";#{self.upc_amount}"
        data << (self.alt_amount.nil? ? "" : ", #{self.alt_amount}")
      end
      data << (self.sd.nil? ? ";" : ";#{self.sd}")
      data << (self.tran_code.nil? ? ";" : ";#{self.tran_code}")
      data << (self.approval_code.nil? ? ";" : ";#{self.approval_code}")
      data << ";"
      data
    end

    #MerchantId;TerminalId;PurchaseTime;OrderId,Delay;CurrencyId,AltCurrencyId;Amount,AltAmount;SessionData(SD);
    def make_request_data
      data = ""
      data << (self.merchant_id.nil? ? "" : "#{self.merchant_id}")
      data << (self.terminal_id.nil? ? ";" : ";#{self.terminal_id}")
      data << ";#{self.purchase_time}"
      data << ";#{self.order_id}"
      data << (self.delay.nil? ? "" : ", #{self.delay}")
      if self.currency_id.nil?
        data << ";"
      else
        data << ";#{self.currency_id}"
        data << (self.alt_currency_id.nil? ? "" : ", #{self.alt_currency_id}")
      end
      if self.amount.nil?
        data << ";"
      else
        data << ";#{self.upc_amount}"
        data << (self.alt_amount.nil? ? "" : ", #{self.alt_amount}")
      end
      data << (self.sd.nil? ? ";" : "#{;self.sd}")
      data << ";"
      data
    end
  end
end